using System;
using System.Collections.Generic;
using System.Threading;
using System.Text;

namespace LD10Tanks
{
    /// <summary>
    /// A class that runs the marching cubes background on another thread.
    /// </summary>
    public class BackgroundCubes
    {
        private AutoResetEvent m_evt_new_work = new AutoResetEvent(false);
        private AutoResetEvent m_evt_work_done = new AutoResetEvent(false);

        private volatile bool m_exit = false;
        private volatile bool m_job_started;
        private volatile bool m_job_running;

        private EvalLevel m_level;
        private MarchingCubes m_tri_gen;

        public FastList<VertexPositionNormalColor> vertices = new FastList<VertexPositionNormalColor>();
        public FastList<int> triangles = new FastList<int>();

        public bool JobStarted
        {
            get { return m_job_started; }
        }
        public bool JobRunning
        {
            get { return m_job_running; }
        }

        public BackgroundCubes(MarchingCubes tri_gen)
        {
            m_level = new EvalLevel();
            m_tri_gen = tri_gen;

            // Start the other thread
            Thread t = new Thread(Run);
            t.Start();
        }

        public void Close()
        {
            SyncJob();
            m_exit = true;
            m_evt_new_work.Set();
        }

        public void StartJob(EvalLevel level)
        {
            // Sync old job
            SyncJob();

            // Copy the level data
            m_level.CopyFrom(level);

            // Start a new job
            m_job_started = true;
            m_job_running = true;
            m_evt_new_work.Set();
        }

        public void SyncJob()
        {
            if (!m_job_started)
                return;
            m_evt_work_done.WaitOne();
            m_job_started = false;

            // Copy the newest data
            vertices.CopyFrom(m_tri_gen.Vertices);
            triangles.CopyFrom(m_tri_gen.Triangles);
        }

        private void Run()
        {
            while (!m_exit)
            {
                // Wait for work
                m_evt_new_work.WaitOne();

                // Exit?
                if (m_exit)
                    break;

                // Do the work
                m_tri_gen.Generate(m_level);

                // Signal job is done
                m_job_running = false;
                m_evt_work_done.Set();
            }
        }
    }
}
