using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TankLevelEditor
{
    public partial class MainEditor : Form
    {
        public LevelList m_levels;

        public MainEditor()
        {
            InitializeComponent();

            // Load the levels
            m_levels = new LevelList();

            // Add all levels to the list
            foreach (Level level in m_levels.levels)
            {
                listBoxLevels.Items.Add(level.Name);
            }
        }

        private void listBoxLevels_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listBoxLevels.SelectedIndex < 0)
            {
                levelEditor.EditLevel = null;
                propertyGrid.SelectedObject = null;
                labelCurrentLevel.Text = "-";

                buttonDeleteLevel.Enabled = false;
                buttonLevelDown.Enabled = false;
                buttonLevelUp.Enabled = false;

                buttonNewWall.Enabled = false;
                buttonNewTank.Enabled = false;
                buttonRemoveWallTank.Enabled = false;
            }
            else
            {
                levelEditor.EditLevel = m_levels.levels[listBoxLevels.SelectedIndex];
                propertyGrid.SelectedObject = levelEditor.EditLevel;
                labelCurrentLevel.Text = levelEditor.EditLevel.Name;

                buttonDeleteLevel.Enabled = true;
                buttonLevelDown.Enabled = (listBoxLevels.SelectedIndex < (m_levels.levels.Count - 1));
                buttonLevelUp.Enabled = (listBoxLevels.SelectedIndex > 0);

                buttonNewWall.Enabled = true;
                buttonNewTank.Enabled = true;
                buttonRemoveWallTank.Enabled = false;
            }
        }

        private void levelEditor_ActiveItemChangedEvent(object item)
        {
            propertyGrid.SelectedObject = item;

            buttonRemoveWallTank.Enabled = (item != null);
        }

        private void buttonNewLevel_Click(object sender, EventArgs e)
        {
            Level level = new Level();
            m_levels.levels.Add(level);

            listBoxLevels.Items.Add(level.Name);
            listBoxLevels.SelectedIndex = listBoxLevels.Items.Count - 1;
        }

        private void levelEditor_LevelChangedEvent(object item)
        {
            propertyGrid.SelectedObject = item;
        }

        private void propertyGrid_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
        {
            if (propertyGrid.SelectedObject == null)
                return;

            if (propertyGrid.SelectedObject is Level)
            {
                listBoxLevels.Items[listBoxLevels.SelectedIndex] = ((Level)propertyGrid.SelectedObject).Name;
            }

            if (propertyGrid.SelectedObject is LevelItem)
            {
                levelEditor.Invalidate();
            }
        }

        private void buttonDeleteLevel_Click(object sender, EventArgs e)
        {
            if (listBoxLevels.SelectedIndex < 0)
                return;

            int idx = listBoxLevels.SelectedIndex;

            listBoxLevels.SelectedIndex = -1;
            m_levels.levels.RemoveAt(idx);
            listBoxLevels.Items.RemoveAt(idx);
        }

        private void buttonRemoveWallTank_Click(object sender, EventArgs e)
        {
            LevelItem item = levelEditor.ActiveItem;
            if (item == null)
                return;

            if (item is Tank)
                levelEditor.EditLevel.tanks.Remove((Tank)item);
            if (item is Wall)
                levelEditor.EditLevel.walls.Remove((Wall)item);

            levelEditor.ActiveItem = null;
        }

        private void buttonNewWall_Click(object sender, EventArgs e)
        {
            if (levelEditor.EditLevel == null)
                return;

            Wall item = new Wall();
            levelEditor.EditLevel.walls.Add(item);
            levelEditor.ActiveItem = item;
        }

        private void buttonNewTank_Click(object sender, EventArgs e)
        {
            if (levelEditor.EditLevel == null)
                return;

            Tank item = new Tank();
            levelEditor.EditLevel.tanks.Add(item);
            levelEditor.ActiveItem = item;
        }

        private void buttonLevelUp_Click(object sender, EventArgs e)
        {
            int idx = listBoxLevels.SelectedIndex;
            if (idx < 0)
                return;

            Level l1 = m_levels.levels[idx];
            m_levels.levels[idx] = m_levels.levels[idx - 1];
            m_levels.levels[idx - 1] = l1;

            listBoxLevels.Items[idx] = m_levels.levels[idx].Name;
            listBoxLevels.Items[idx - 1] = m_levels.levels[idx - 1].Name;
            listBoxLevels.SelectedIndex = idx - 1;
        }

        private void buttonLevelDown_Click(object sender, EventArgs e)
        {
            int idx = listBoxLevels.SelectedIndex;
            if (idx < 0)
                return;

            Level l1 = m_levels.levels[idx];
            m_levels.levels[idx] = m_levels.levels[idx + 1];
            m_levels.levels[idx + 1] = l1;

            listBoxLevels.Items[idx] = m_levels.levels[idx].Name;
            listBoxLevels.Items[idx + 1] = m_levels.levels[idx + 1].Name;
            listBoxLevels.SelectedIndex = idx + 1;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            m_levels.Save();
        }
    }
}